

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <time.h>

#include "transfer.h"
#include "dataspy.h"

void checkDATAbuffer(int, char*, int);
void printbufferS(char*, int);
void printbufferL(char*, int);

//    24 byte data block header - used for data blocks in shared memory and data blocks on disc or tape.

typedef struct s_data_header {
    char header_id[8];
    int header_sequence;
    short  header_stream;
    short  header_tape;
    short  header_MyEndian;
    short  header_DataEndian;
    int header_dataLen;
} DATA_HEADER;


void Usage(char *progname) {
    fprintf(stderr,"Usage\n%s -n server -i [id] -s [buffer size]\n",progname);
    fprintf(stderr,"\tid is the source data shared memory area ID to use\n");
    exit(1);
}
        
int main(int argc, char **argv) {

    char Buffer [64+(64*1024)];    /*  a 64Kbyte data buffer + a bit extra to handle block headers  */
    DATA_HEADER data_header;
    int BufferLength;
    int DataLength;

    int id=0;
    unsigned int stream = 1;
    int verbose=0;
    int s;
    int i;
    int BufferSize=64*1024;

    int DelayTicks=10;

    struct timespec time_request;


    if (argc >1) {
        for(i=1;i <argc;i++) {
            if ( (argv[i][0] == '-') || (argv[i][0] == '/') ) {
                switch(tolower(argv[i][1])) {
                    case 'i':
                        id = atoi(argv[++i]);
                        break;
                    case 's':
                        BufferSize = atoi(argv[++i])*1024;
                        break;
                    case 'v':
                        verbose = 1;
                        break;
                    default:
                        Usage(argv[0]);
                        break;
                }
            }
            else
                Usage(argv[0]);
        }
    }


    i = dataSpyOpen (id);
    
    for (;;)
    {

/*    obtain a data buffer from shared memory. This will have a 24 byte header */

       DataLength = dataSpyRead (id, (char*)&Buffer[sizeof(HEADER)-sizeof(DATA_HEADER)], BufferSize);  /* read into buffer at offset 8 bytes; real data starts at 8+24 = 32  */
       if (verbose) printf("read done with code=%d\n",DataLength);

       if (DataLength > 0) {

              memcpy (data_header.header_id, (char*)&Buffer[sizeof(HEADER)-sizeof(DATA_HEADER)], sizeof(DATA_HEADER));
              BufferLength = data_header.header_dataLen;   /* the actual application data less headers  */
              if (verbose) printf("buffer length=%d\n",BufferLength);

              printbufferS(&Buffer[sizeof(HEADER)],BufferLength);

              checkDATAbuffer(stream, &Buffer[sizeof(HEADER)], BufferLength);

//     printbufferL(&Buffer[sizeof(HEADER)],BufferLength);
         
       } else {
           if (DelayTicks) {

               time_request.tv_sec = DelayTicks/1000;
               time_request.tv_nsec = (DelayTicks - (time_request.tv_sec * 1000)) * 1000000;
               i = nanosleep(&time_request, NULL);
            }
       }
    }
}

#include "checkDatabuffer.c"

